/*****************************************
  xjump version 2

  record.c   _L^

  (C) July 16, 1997  ROYALPANDA
*****************************************/

/* modified by shinichiro.h in 020820 */


#include<stdio.h>
#include<ctype.h>
#include<errno.h>
#include<stdlib.h>
#include<string.h>

#ifdef UNIX
# include<unistd.h>
# include<pwd.h>
# include<sys/types.h>
# include<sys/file.h>
#endif

#include"record.h"


static char* LoadFilename;

/* Xy[Xǂݔ΂ */

static int skipspace( FILE *fp )
{
  int c;

  for(;;){
    c = fgetc(fp);
    if( c == EOF )
      return -1;
    if( c == '\n' )
      return -1;
    if( !isspace(c) ){
      ungetc(c,fp);
      return 0;
    }
  }
}


/* lǂݍ */

static int read_num( FILE *fp )
{
  int c,i;
  int err;

  if( skipspace( fp ) )
    return -1;

  i = 0;
  err = 1;

  for(;;){
    c = fgetc( fp );
    if( isdigit( c ) ){
      i = i * 10 + c - '0';
      err = 0;
    }else
      break;
  }
  ungetc( c,fp );

  if( err )
    return -1;
  else
    return i;
}

/* O̓ǂݍ */

static void read_name( char *buf, FILE *fp )
{
  int l,c;

  if( skipspace( fp ) ){
    *buf = '\0';
    return;
  }

  for( l = 0 ; l < 31 ; l++ ){
    c = fgetc( fp );
    if( c < 0 || iscntrl( c ) ){
      ungetc( c,fp );
      break;
    }
    *buf++ = c;
  }
  *buf = '\0';
}


/* s܂œǂݔ΂ */

static void void_line( FILE *fp )
{
  int c;

  while( ( c = fgetc(fp) ) != '\n' && c != EOF )
    ;
}


#ifdef UNIX

/* UID烊Al[o */

static char *get_name( int uid, char *name )
{
  struct passwd *pw;
  int i;

  if( ( pw = getpwuid( uid ) ) == NULL )
     name[0] = '\0';

  else{

    i = 0;
    while( i < 31 && pw->pw_gecos[i] != '\0' && pw->pw_gecos[i] != ',' ){
      name[i] = pw->pw_gecos[i];
      i++;
    }

    if( i > 0 )
      name[i] = '\0';
    else
      strcpy( name,pw->pw_name );

  }

  return name;
}

#endif /* UNIX */


/*@R[ĥPGg[ǂݍ݁@*/

static int read_entry( FILE *fp, record_t *rec )
{
  if( ( rec->score = read_num( fp ) ) < 0 )
    return -1;
  if( ( rec->second = read_num( fp ) ) < 0 )
    return -1;
  read_name( rec->name,fp );

  return 0;
}


/* R[hǂ݂ */  

static void read_record( FILE *fp )
{
  int i;

  i = 0;

  while( !feof( fp ) && i < RECORD_ENTRY ){
    if( read_entry( fp,&Record[i] ) == 0 )
      i++;
    void_line( fp );
  }
  Record_entry = i;
}



/* G[ */

static void error( void )
{
  perror( Myname );
  fprintf( stderr,"%s: do not record high-score.\n",Myname);

  Record_entry = -1;
}


/* R[h֌W̏ */

void init_record( void )
{
  FILE *fp;

  Record_entry = -1;

#ifdef UNIX
  LoadFilename = RECORD_FILE;
  if ((fp = fopen(LoadFilename, "r")) == NULL) {
	LoadFilename = "record.dat";
	if ((fp = fopen(LoadFilename, "r")) == NULL)
	{
	  error();
	  return;
	}
  }
#else
  LoadFilename = "record.dat";
  if ((fp = fopen(LoadFilename, "r")) == NULL) {
	error();
	return;
  }
#endif

#ifdef UNIX
  flock( fileno(fp),LOCK_EX );
#endif

  read_record( fp );
  fclose( fp );
}


/* \[gr֐ */

static int sort_cmp(const void* v1, const void* v2 )
{
  const record_t* r1 = (const record_t*)v1;
  const record_t* r2 = (const record_t*)v2;

  if( r1->score > r2->score )
    return -1;
  else if( r1->score < r2->score )
    return 1;

  if( r1->second > r2->second )
    return 1;
  else if( r1->second < r2->second )
    return -1;

  else if( r1->rank > r2->rank )
    return 1;
  else
    return -1;
}



/* \[g */

static void sort_record( void )
{
  int i;

  for( i = 0 ; i < Record_entry ; i++ )
    Record[i].rank = i;

  qsort( Record,Record_entry,sizeof(record_t),sort_cmp );
}



/* R[hL^ */

void save_record( int sc, int second )
{
  FILE *fp;
  int i,r;
  char name[32];

#ifdef UNIX
  int uid;
#endif

  if( Record_entry == -1 )
    return ;

  if( (fp = fopen(LoadFilename,"r")) == NULL ){
    error();
    return;
  }

#ifdef UNIX
  flock( fileno(fp),LOCK_EX );
#endif

  read_record( fp );
  fclose(fp);

  r = -1;

#ifdef UNIX
  uid = (int)getuid();
  get_name(uid, name);
#else
  fp = fopen("name.dat", "r");
  if (fp == NULL) strncpy(name, "sjump", 7);
  else {
	read_name(name, fp);
	fclose(fp);
  }
#endif

  for (i = 0; i < Record_entry; i++ ) {
    if (strncmp(name, Record[i].name, 31) == 0) {
      r = i;
      break;
    }
  }

  if( r == -1 ){
    if( Record_entry < RECORD_ENTRY ){
      r = Record_entry++;
      Record[r].score = 0;
    }
	else
      r = RECORD_ENTRY - 1;
  }

  if( sc > Record[r].score ){
    Record[r].score = sc;
	Record[r].second = second;
	strncpy(Record[r].name, name, 31);

    sort_record();

	if( (fp = fopen(LoadFilename,"w")) == NULL ){
	  error();
	  return;
	}

    for( i = 0 ; i < Record_entry ; i++ )
      fprintf( fp,"%d %d %s\n",
	      Record[i].score,Record[i].second,Record[i].name );

    fclose( fp );
    return;
  }
}
